/* src\file_conversor\.web\static\css\base.css */

:root {
    --bulma-navbar-height: 2.85rem;
    --bulma-status-bar-height: 2rem;

    --bulma-navbar-border-color: #c7c7c7;
    --bulma-hover-border-color: #828282;
    --bulma-tab-border-color: #c0bfbf;
    --bulma-modal-card-head-background-color: var(--bulma-scheme-main);
    --bulma-footer-background-color: var(--bulma-scheme-main);

    --bulma-main-app-margin: 20px;
    --bulma-shadow: 0px 0px 15px 2px rgba(70, 70, 70, 0.4);
}

html,
body {
    height: 100%;
    width: 100%;
    margin: 0;
    padding: 0;
}


/* Font smoothing for better text rendering */
html,
body,
input,
button {
    text-rendering: geometricPrecision;
    /* Other options: geometricPrecision, optimizeSpeed, optimizeLegibility */
    -webkit-font-smoothing: antialiased;
    /* Better contrast (thin, crisp text) */
    -moz-osx-font-smoothing: grayscale;
    /* Firefox macOS equivalent */
}

html.has-status-bar-fixed-bottom,
body.has-status-bar-fixed-bottom {
    padding-bottom: var(--bulma-status-bar-height);
}

#main-app {
    margin-top: var(--bulma-main-app-margin);
    margin-bottom: var(--bulma-main-app-margin);
}

.is-64x64 {
    width: 64px !important;
    height: 64px !important;
}

.is-48x48 {
    width: 48px !important;
    height: 48px !important;
}

.is-32x32 {
    width: 32px !important;
    height: 32px !important;
}

.is-full-width {
    width: 100% !important;
}

.is-full-height {
    height: 100% !important;
}

.has-shadow {
    box-shadow: var(--bulma-shadow) !important;
}

.has-border {
    border: solid 1px var(--bulma-navbar-border-color) !important;
}

/* CURSORS */
.is-cursor-auto {
    cursor: auto !important;
}

.is-cursor-copy {
    cursor: copy !important;
}

.is-cursor-pointer {
    cursor: pointer !important;
}

.is-cursor-grab {
    cursor: grab !important;
}

.is-cursor-grabbing {
    cursor: grabbing !important;
}

.is-cursor-progress {
    cursor: progress !important;
}

.is-cursor-wait {
    cursor: wait !important;
}

/* NAVBAR */
body nav.navbar.is-fixed-top {
    height: var(--bulma-navbar-height);
}

body nav.navbar.is-fixed-top .navbar-menu {
    flex-grow: 0;
}

body nav.navbar.is-fixed-top.has-border {
    border-top: 1px solid var(--bulma-navbar-border-color);
    border-bottom: 1px solid var(--bulma-navbar-border-color);
    box-shadow: var(--bulma-shadow);
}

body nav.navbar .navbar-menu .navbar-item,
body nav.navbar .navbar-menu .navbar-item a {
    color: var(--bulma-navbar-item-color);
}

body nav.navbar .navbar-menu .navbar-dropdown {
    border: solid 1px var(--bulma-navbar-border-color);
    border-top: none;
}

@media screen and (max-width: 1023px) {
    body nav.navbar.is-fixed-top>.navbar-menu {
        display: block !important;
        max-height: 0;
        padding-top: 0;
        padding-bottom: 0;
        opacity: 0;
        visibility: hidden;
        overflow: hidden;
        transition: all 0.35s ease;
    }

    body nav.navbar.is-fixed-top>.navbar-menu.is-active {
        max-height: calc(100vh - var(--bulma-navbar-height));
        padding-top: 8px;
        padding-bottom: 8px;
        opacity: 1;
        visibility: visible;
    }
}

/* MODAL */
body .modal-card,
body .modal-content {
    min-width: 40rem !important;
    width: auto !important;
    max-width: 80vw !important;
}

body .modal-card-head,
body .modal-card-foot {
    background-color: var(--bulma-modal-card-head-background-color);
}

body .modal-card-foot {
    border-top: solid 1px var(--bulma-navbar-border-color);
}

/* CARDS */
body .card {
    border: solid 1px var(--bulma-navbar-border-color);
    box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
    transition: box-shadow 0.3s ease, transform 0.3s ease, border 0.3s ease;
}

body .card:hover {
    border: solid 1px var(--bulma-hover-border-color);
    box-shadow: 0px 8px 14px 2px rgba(0, 0, 0, 0.4);
    transform: translateY(-6px);
    z-index: 20;
}

/* CELLS */
body .cell,
body .cell>a,
body .cell .card,
body .cell .card>.card-content {
    height: 100%;
}

body .cell .card>.card-content {
    display: flex;
    flex-direction: column;
    /* justify-content: center; */
}

/* TABS */
body .tabs .tab-headers a {
    border: solid 1px var(--bulma-tab-border-color);
}

body .tabs .tab-headers a:hover {
    border: solid 1px var(--bulma-hover-border-color);
}

/* input */
body input.input,
body textarea.textarea {
    --bulma-input-placeholder-color: #878787;
}

body input[type="checkbox"] {
    border: var(--bulma-navbar-border-color);
}

/* select */
body .select select[multiple] {
    overflow-x: auto;
    max-width: 100%;
}

/* buttons */
body button.button {
    border: solid 1px var(--bulma-navbar-border-color) !important;
}

body button.button:hover {
    border: solid 1px var(--bulma-hover-border-color) !important;
}

/* FOOTER */
body #footer {
    padding: 3rem;
    padding-bottom: 4rem;
    background-color: var(--bulma-footer-background-color);
}

/* STATUS BAR */
body .status-bar.is-fixed-bottom {
    position: fixed;
    bottom: 0;
    left: 0;
    width: 100%;
    height: var(--bulma-status-bar-height);
    color: var(--bulma-navbar-item-color);
    background-color: var(--bulma-footer-background-color);
    border-top: solid 1px var(--bulma-navbar-border-color);
    border-bottom: solid 1px var(--bulma-navbar-border-color);
    text-align: left;
    box-shadow: var(--bulma-shadow);
    z-index: 30;
}