# src\file_conversor\cli\xls\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "xls"

# SUBCOMMANDS
CONVERT_NAME = "convert"
