# src/file_conversor/backend/gui/_components/padding.py

from re import M
from file_conversor.utils.dominate_utils import *


class PaddingAll:
    P_AUTO = "p-auto"
    P_0 = "p-0"
    P_1 = "p-1"
    P_2 = "p-2"
    P_3 = "p-3"
    P_4 = "p-4"
    P_5 = "p-5"
    P_6 = "p-6"


class PaddingX:
    P_AUTO = "px-auto"
    P_0 = "px-0"
    P_1 = "px-1"
    P_2 = "px-2"
    P_3 = "px-3"
    P_4 = "px-4"
    P_5 = "px-5"
    P_6 = "px-6"


class PaddingY:
    P_AUTO = "py-auto"
    P_0 = "py-0"
    P_1 = "py-1"
    P_2 = "py-2"
    P_3 = "py-3"
    P_4 = "py-4"
    P_5 = "py-5"
    P_6 = "py-6"


class PaddingLeft:
    P_AUTO = "pl-auto"
    P_0 = "pl-0"
    P_1 = "pl-1"
    P_2 = "pl-2"
    P_3 = "pl-3"
    P_4 = "pl-4"
    P_5 = "pl-5"
    P_6 = "pl-6"


class PaddingRight:
    P_AUTO = "pr-auto"
    P_0 = "pr-0"
    P_1 = "pr-1"
    P_2 = "pr-2"
    P_3 = "pr-3"
    P_4 = "pr-4"
    P_5 = "pr-5"
    P_6 = "pr-6"


class PaddingTop:
    P_AUTO = "pt-auto"
    P_0 = "pt-0"
    P_1 = "pt-1"
    P_2 = "pt-2"
    P_3 = "pt-3"
    P_4 = "pt-4"
    P_5 = "pt-5"
    P_6 = "pt-6"


class PaddingBottom:
    P_AUTO = "pb-auto"
    P_0 = "pb-0"
    P_1 = "pb-1"
    P_2 = "pb-2"
    P_3 = "pb-3"
    P_4 = "pb-4"
    P_5 = "pb-5"
    P_6 = "pb-6"


__all__ = [
    'PaddingAll',
    'PaddingX',
    'PaddingY',
    'PaddingTop',
    'PaddingBottom',
    'PaddingLeft',
    'PaddingRight',
]
